shoutbox = {
	main = "Shoutbox",
	version = "v1.0d #004",
	about = [[
		Shoutbox is used for nationalistic and piracy messaging in 100; after you say a key phrase, the plugin will add on a random message from one you've stored.
	]],
	type = "DEFAULT",
	action = function() print("Not available on QuasarUI!") end,
}

if QUASAR ~= nil then --test to see if the QuasarUI is set up
	RegisterPlugin(shoutbox) --Registers the plugin into QuasarUI's plugin menu
	return --prevents this plugin from actually doing anything - its meant for DefaultUI, after all.
end



shoutbox.lastPhraseIndex = 1

shoutbox.data = {
	['test'] = "shoutbox", --to make sure systemnotes file exists as shoutbox data
	['active'] = "OFF", --is the plugin active
	['key'] = "Glory!", --what triggers the plugin
	['phrase'] = { --the set of phrases the plugin will push to chat
		[1] = "Glory to Vendetta Online!",
	},
}

local function cleanShoutboxData()
	local counter = 0
	for key, value in pairs(shoutbox.data['phrase']) do
		counter = counter + 1
		shoutbox.data['phrase'][counter] = value
	end
	if #shoutbox.data['phrase'] > counter then
		local numLeftToPurge = #shoutbox.data['phrase'] - counter
		for a=1, numLeftToPurge do
			shoutbox.data['phrase'][#shoutbox.data['phrase']] = nil
		end
	end
end

local function getUserDataTable()
	
	local userDataTable = unspickle(LoadSystemNotes(15000))
	local isValid = false
	if userDataTable ~= nil then
		if userDataTable['test'] == "shoutbox" then
			isValid = true
		end
	end
	
	if isValid == true then
		shoutbox.data = userDataTable
	end
	
	cleanShoutboxData()
	
	shoutbox.ui.phraseTable:emptyList()
	shoutbox.ui.phraseTable:buildList()
	
end







shoutbox.ui = {}

local function createShoutboxEditor()
	
	local keyEditor = iup.text {
		expand = "HORIZONTAL",
		value = shoutbox.data['key'],
	}
	
	local keyViewer = iup.label {
		title = shoutbox.data['key'],
	}
	
	local phraseEditor = iup.text {
		expand = "HORIZONTAL",
		value = "",
	}
	
	local shoutboxToggle = iup.stationtoggle {
		title = " Activate shoutbox",
		value = shoutbox.data['active'],
		action = function(self)
			shoutbox.data['active'] = self.value
			SaveSystemNotes(spickle(shoutbox.data), 15000)
		end,
	}
	
	shoutbox.ui.phraseTable = iup.stationsublist {
		MULTILINE = "YES",
		EXPAND = "YES",
		BORDER = "YES",
		BGCOLOR = "0 0 0 100 *",
		action = function(self, itemText, listIndex, clickValue)
			if clickValue == 1 then
				shoutbox.lastPhraseIndex = listIndex
			end
		end,
	}
	
	function shoutbox.ui.phraseTable:buildList()
		if #shoutbox.data['phrase'] > 0 then
			for phraseIndex, phraseString in ipairs(shoutbox.data['phrase']) do
				self[phraseIndex] = phraseString
			end
		end
		keyViewer.title = shoutbox.data['key']
		shoutboxToggle.value = shoutbox.data['active']
	end
	
	shoutbox.ui.phraseTable:buildList()
	
	function shoutbox.ui.phraseTable:emptyList()
		for phraseIndex, _ in ipairs(shoutbox.data['phrase']) do
			self[phraseIndex] = nil
		end
	end
	
	function shoutbox.ui.phraseTable:removeItem()
		if #shoutbox.data['phrase'] > 1 then
			if #shoutbox.data['phrase'] >= shoutbox.lastPhraseIndex then
				shoutbox.ui.phraseTable:emptyList()
				shoutbox.data['phrase'][shoutbox.lastPhraseIndex] = nil
				cleanShoutboxData()
				shoutbox.ui.phraseTable:buildList()
				SaveSystemNotes(spickle(shoutbox.data), 15000)
			end
		end
	end
	
	function shoutbox.ui.phraseTable:addItem()
		if string.len(phraseEditor.value) > 2 then
			shoutbox.ui.phraseTable:emptyList()
			shoutbox.data['phrase'][#shoutbox.data['phrase'] + 1] = phraseEditor.value
			shoutbox.ui.phraseTable:buildList()
			SaveSystemNotes(spickle(shoutbox.data), 15000)
		end
	end
	
	local diag = iup.dialog {
		topmost = "YES",
		size = HUDSize(0.5, 0.75),
		bgcolor = "0 0 0 200 *",
		alignment = "ACENTER",
		iup.vbox {
			margin = "5x2",
			iup.hbox {
				iup.label {
					title = "Shoutbox configuration",
					font = Font.Large,
				},
				iup.fill { },
				iup.label {
					title = shoutbox.version,
				},
			},
			shoutboxToggle,
			iup.label { --spacer
				title = "",
			},
			iup.hbox {
				iup.label {
					title = "trigger key: ",
				},
				keyViewer,
				iup.fill { },
				iup.stationbutton {
					title = "update trigger word",
					action = function()
						if string.len(keyEditor.value) > 2 then
							shoutbox.data['key'] = keyEditor.value
							keyViewer.title = keyEditor.value
							SaveSystemNotes(spickle(shoutbox.data), 15000)
							iup.Refresh(keyViewer)
						end
					end,
				},
			},
			keyEditor,
			iup.label { --spacer
				title = "",
			},
			iup.label {
				title = "Possible phrases to shout:",
			},
			iup.vbox {
				shoutbox.ui.phraseTable,
				iup.hbox {
					phraseEditor,
					iup.stationbutton {
						title = "Add new phrase",
						action = function()
							shoutbox.ui.phraseTable:addItem()
							phraseEditor.value = ""
						end,
					},
					iup.stationbutton {
						title = "Remove selected phrase",
						action = function()
							shoutbox.ui.phraseTable:removeItem()
						end,
					},
				},
			},
			iup.label { --spacer
				title = "",
			},
			iup.hbox {
				iup.stationbutton {
					title = "Reset data",
					action = function()
						shoutbox.data = {
							['test'] = "shoutbox", --to make sure systemnotes file exists as shoutbox data
							['active'] = "OFF", --is the plugin active
							['key'] = "Glory!", --what triggers the plugin
							['phrase'] = { --the set of phrases the plugin will push to chat
								[1] = "Glory to Vendetta Online!",
							},
						}
						
						shoutbox.ui.phraseTable:emptyList()
						shoutbox.ui.phraseTable:buildList()
					end,
				},
				iup.fill { },
				iup.stationbutton {
					title = "Close configuration",
					action = function()
						shoutbox.ui.diag:hide()
					end,
				},
			},
		},
	}
	
	shoutbox.ui.diag = diag
	
end

createShoutboxEditor()

RegisterUserCommand("shoutbox", function() shoutbox.ui.diag:show() end)

local openShoutboxButton = iup.stationbutton {
	cx = (gkinterface.GetXResolution()/2) - 95,
	cy = gkinterface.GetYResolution()/2 + 48,
	title = "Shoutbox Config",
	action = function()
		shoutbox.ui.diag:show()
	end,
}

iup.Append(OptionsDialog[1][1], openShoutboxButton)



local shoutboxTimer = Timer()

shoutbox.TimerHook = {
	outputTable = {},
}

function shoutbox.TimerHook:newOutput(message)
	self.outputTable[#self.outputTable + 1] = message
	shoutboxTimer:SetTimeout(1000, function() shoutbox.TimerHook:onTimerTick() end)
end

function shoutbox.TimerHook:onTimerTick()
	
	if #self.outputTable > 0 then
		SendChat(self.outputTable[#self.outputTable], "CHANNEL", GetActiveChatChannel())
		self.outputTable[#self.outputTable] = nil
	end
	
	if shoutbox.data['active'] == "ON" then
		shoutboxTimer:SetTimeout(1000, function() shoutbox.TimerHook:onTimerTick() end)
	end
	
end



local function chatHook(_, arg)
	if shoutbox.data['active'] == "ON" then
		if arg.name == GetPlayerName() then
			if arg.msg == shoutbox.data['key'] then
				shoutbox.TimerHook:newOutput(shoutbox.data['phrase'][math.random(#shoutbox.data['phrase'])])
			end
		end
	end
end

RegisterEvent(chatHook, "CHAT_MSG_CHANNEL_ACTIVE")
RegisterEvent(getUserDataTable, "PLAYER_ENTERED_GAME")



















